function [t, w, Lod,Ls]=TPW_Hu_for_Midori6(M,alpha,t0,Dt,End_time,Model,Omega_portion,Loadtype)
%% Semi-analytical solution for TPW on rotating body whose Chandler period is much shorter than its dominant relaxation time (Earth-like).
%% by Haiyang Hu   04-13-2017
%% Output
% t: time step;  
% w: colatitude and longitude of the rotational axis (rad) in the body-fixed frame;
% Lod: colatitude and longitude of the mass anomaly (rad) in the bulge-fixed frame; 
% Ls: length of the day change; 
%% Input 
% M: mass of the massanomaly (kg);
% alpha: colatitude (degree) of the mass anomaly. It is placed at 0 degree langutude. 
% t0: Initial Loading time (ka) for the model to reach its desired hydrostatic state. So large enough for all the modes to relax for a model in hydrostatic equilibrium.
% Dt: step size (ka)
% End_time: simulation length (ka)
% Model: choice of the interior model Love numbers;
% Omega_portion: a scale factor for the rotational speed. Set to 1 for  normal simulation.
% Lodtype: 1 for Heaviside, 2 for Ramp type.
%% Example
% To reproduce figure 4 in [Hu at al, 2017b], type following setings:
% M=2e19;
% alpha=30;
% End_time=1.3e4;
% Model=0;
% 
% [tt1,mm1]=TPW_Hu_for_Midori6(M,alpha,2e3,5,End_time,0,1,1);
% [tt2,mm2]=TPW_Hu_for_Midori6(M,alpha,1e6,5,End_time,0,1,1);

%% Load Love numbers for the chosen model

if Model==1
    data=dlmread('F:\Python_code\Normal Mode\modt_SG6_10_1.txt'); %10km lithosphere high viscosity
elseif Model==2 
    data=dlmread('F:\Python_code\Normal Mode\modt_SG6_10_3.txt'); %10km lithosphere low viscosity
elseif Model==3
    data=dlmread('F:\Python_code\Normal Mode\modt_SG6_20_1.txt'); %20km lithosphere
elseif Model==4
    data=dlmread('F:\Python_code\Normal Mode\modt_SG6_20_3.txt');
elseif Model==5
    data=dlmread('F:\Python_code\Normal Mode\modt_SG6_30_1.txt');
elseif Model==6
    data=dlmread('F:\Python_code\Normal Mode\modt_SG6_30_2.txt');
elseif Model==0
    data=dlmread('F:\Python_code\Normal Mode\modt_SG6.txt');  %Model without lithosphere
elseif Model==10
    data=dlmread('F:\Python_code\Normal Mode\Mars19.txt');   
elseif Model==11
    data=dlmread('F:\Python_code\Normal Mode\Mars20.txt');   
elseif Model==12
    data=dlmread('F:\Python_code\Normal Mode\Mars21.txt'); 
elseif Model==13
    data=dlmread('F:\Python_code\Normal Mode\Mars22.txt');
elseif Model==102
    data=dlmread('F:\Python_code\Normal Mode\modt_Triton'); 
elseif Model==103
    data=dlmread('F:\Python_code\Normal Mode\modt_Triton_10.txt');   
else
    error('unknown type of model')
end
% maxdeg=data(1);
nlayer=data(2);
maxmod=data(3);
data=data(4+nlayer:end,:);
D=1;

Ke=data((maxmod+2)*(D-1)+1,3);
Kf=data((maxmod+2)*(D-1)+maxmod+2,3);
Ki=-data((maxmod+2)*(D-1)+(1:maxmod)+1,3).*data((maxmod+2)*(D-1)+(1:maxmod)+1,2);
Si=data((maxmod+2)*(D-1)+(1:maxmod)+1,2);

%%
GG=6.67384e-11;
ka=1000*365.25*24*3600;

if Model<10 % For Earth SG6 model
    I=7.2457582847984906e+37;
    Omega=2*pi/(24)/60/60*Omega_portion;
    R=6371e3;
elseif Model>100 %Triton
    I=7.0920477e33; %moment of inertia of unperturbed Triton 
    R=1352e3;
    Omega= 2*pi/(((5*24+21)*60+2)*60+53); 
else   % For Mars
    I=2.700761343962756e+36;
    Omega=2*pi/(24)/60/60*Omega_portion/1.025957;
    R=3389.5e3;
end

 Omega0=Omega;
%% Load parameters
alpha=alpha/180*pi;
x=sin(alpha); y=0; z=cos(alpha);
E=M*R^2*[y^2+z^2 -x*y -x*z; -y*x x^2+z^2 -y*z; -z*x -z*y x^2+y^2]; % Load matrix    
dE=0;

%%
t=zeros(1,3000);
t(1)=t0;
w=zeros(3,3000);
w(:,1)=[0,0,Omega]';
Lod=zeros(1,1000);
Lod(1)=1;

Ls=zeros(2,3000);
Ls(:,1)=[alpha,0];

A=zeros(3);
dA=zeros(3);
Ind=1;

while t(Ind)<t0+End_time && Ind<=20000
    if Ind==1
        t(Ind+1)=t(Ind)+Dt/10;
    elseif Ind==2
        t(Ind+1)=t(Ind)+Dt-Dt/10;
    else
         t(Ind+1)=t(Ind)+Dt;
    end
    w(:,Ind+1)=w(:,Ind)*Lod(Ind);
    Coe=Ki./Si.*(exp(t(Ind+1)*Si)-exp((t(Ind+1)-t(1))*Si));
    Coe=sum(Coe);
    A(1,1)=Coe*w(1,1)*w(1,1);
    A(1,2)=Coe*w(1,1)*w(2,1);
    A(1,3)=Coe*w(1,1)*w(3,1);
    A(2,2)=Coe*w(2,1)*w(2,1);
    A(2,3)=Coe*w(2,1)*w(3,1);
    A(3,3)=Coe*w(3,1)*w(3,1);  
    
    Coe=Ki.*(exp(t(Ind+1)*Si)-exp((t(Ind+1)-t(1))*Si));
    Coe=sum(Coe);
    if Ind~=0
        dt=t(Ind+1)-t(Ind);
        dA(1,1)=Coe*w(1,1)*w(1,1)+Ke/dt*(w(1,Ind+1)*(w(1,Ind)-2*w(1,Ind+1))+w(1,Ind)*w(1,Ind+1));
        dA(1,2)=Coe*w(1,1)*w(2,1)+Ke/dt*(w(1,Ind+1)*(w(2,Ind)-2*w(2,Ind+1))+w(1,Ind)*w(2,Ind+1));
        dA(1,3)=Coe*w(1,1)*w(3,1)+Ke/dt*(w(1,Ind+1)*(w(3,Ind)-2*w(3,Ind+1))+w(1,Ind)*w(3,Ind+1));
        dA(2,2)=Coe*w(2,1)*w(2,1)+Ke/dt*(w(2,Ind+1)*(w(2,Ind)-2*w(2,Ind+1))+w(2,Ind)*w(2,Ind+1));
        dA(2,3)=Coe*w(2,1)*w(3,1)+Ke/dt*(w(2,Ind+1)*(w(3,Ind)-2*w(3,Ind+1))+w(2,Ind)*w(3,Ind+1));
        dA(3,3)=Coe*w(3,1)*w(3,1)+Ke/dt*(w(3,Ind+1)*(w(3,Ind)-2*w(3,Ind+1))+w(3,Ind)*w(3,Ind+1));
    else
        dA(1,1)=0;
        dA(1,2)=0;
        dA(1,3)=0;
        dA(2,2)=0;
        dA(2,3)=0;
        dA(3,3)=0;
    end
    for ii=1:maxmod
        if Ind>1
            temp=wwn(t(Ind+1),t(1:Ind-1),t(2:Ind),w(1,1:Ind-1),w(1,2:Ind),w(1,1:Ind-1),w(1,2:Ind),Ki(ii),Si(ii));
            A(1,1)=A(1,1)+sum(temp);
            temp=wwn(t(Ind+1),t(1:Ind-1),t(2:Ind),w(1,1:Ind-1),w(1,2:Ind),w(2,1:Ind-1),w(2,2:Ind),Ki(ii),Si(ii));
            A(1,2)=A(1,2)+sum(temp);
            temp=wwn(t(Ind+1),t(1:Ind-1),t(2:Ind),w(1,1:Ind-1),w(1,2:Ind),w(3,1:Ind-1),w(3,2:Ind),Ki(ii),Si(ii));
            A(1,3)=A(1,3)+sum(temp);
            temp=wwn(t(Ind+1),t(1:Ind-1),t(2:Ind),w(2,1:Ind-1),w(2,2:Ind),w(2,1:Ind-1),w(2,2:Ind),Ki(ii),Si(ii));
            A(2,2)=A(2,2)+sum(temp);
            temp=wwn(t(Ind+1),t(1:Ind-1),t(2:Ind),w(2,1:Ind-1),w(2,2:Ind),w(3,1:Ind-1),w(3,2:Ind),Ki(ii),Si(ii));
            A(2,3)=A(2,3)+sum(temp);
            temp=wwn(t(Ind+1),t(1:Ind-1),t(2:Ind),w(3,1:Ind-1),w(3,2:Ind),w(3,1:Ind-1),w(3,2:Ind),Ki(ii),Si(ii));
            A(3,3)=A(3,3)+sum(temp);
            
            temp=dwwn(t(Ind+1),t(1:Ind-1),t(2:Ind),w(1,1:Ind-1),w(1,2:Ind),w(1,1:Ind-1),w(1,2:Ind),Ki(ii),Si(ii));
            dA(1,1)=dA(1,1)+sum(temp);
            temp=dwwn(t(Ind+1),t(1:Ind-1),t(2:Ind),w(1,1:Ind-1),w(1,2:Ind),w(2,1:Ind-1),w(2,2:Ind),Ki(ii),Si(ii));
            dA(1,2)=dA(1,2)+sum(temp);
            temp=dwwn(t(Ind+1),t(1:Ind-1),t(2:Ind),w(1,1:Ind-1),w(1,2:Ind),w(3,1:Ind-1),w(3,2:Ind),Ki(ii),Si(ii));
            dA(1,3)=dA(1,3)+sum(temp);
            temp=dwwn(t(Ind+1),t(1:Ind-1),t(2:Ind),w(2,1:Ind-1),w(2,2:Ind),w(2,1:Ind-1),w(2,2:Ind),Ki(ii),Si(ii));
            dA(2,2)=dA(2,2)+sum(temp);
            temp=dwwn(t(Ind+1),t(1:Ind-1),t(2:Ind),w(2,1:Ind-1),w(2,2:Ind),w(3,1:Ind-1),w(3,2:Ind),Ki(ii),Si(ii));
            dA(2,3)=dA(2,3)+sum(temp);
            temp=dwwn(t(Ind+1),t(1:Ind-1),t(2:Ind),w(3,1:Ind-1),w(3,2:Ind),w(3,1:Ind-1),w(3,2:Ind),Ki(ii),Si(ii));
            dA(3,3)=dA(3,3)+sum(temp);
        end
    end

    
    
    AA=zeros(3);
    dAA=zeros(3);

    Improvement=100;
    temp_v=zeros(3,1);
    Iter=1;
    
    while Improvement>1e-5 && Iter<1500
        AA(1,1)=A(1,1)+Ke*w(1,Ind+1)*w(1,Ind+1);
        AA(1,2)=A(1,2)+Ke*w(1,Ind+1)*w(2,Ind+1);
        AA(1,3)=A(1,3)+Ke*w(1,Ind+1)*w(3,Ind+1);
        AA(2,2)=A(2,2)+Ke*w(2,Ind+1)*w(2,Ind+1);
        AA(2,3)=A(2,3)+Ke*w(2,Ind+1)*w(3,Ind+1);
        AA(3,3)=A(3,3)+Ke*w(3,Ind+1)*w(3,Ind+1);
        
        dt=t(Ind+1)-t(Ind);
        if Ind~=1
        dAA(1,1)=dA(1,1)+Ke/dt*(w(1,Ind+1)*(w(1,Ind)-2*w(1,Ind+1))+w(1,Ind)*w(1,Ind+1));
        dAA(1,2)=dA(1,2)+Ke/dt*(w(1,Ind+1)*(w(2,Ind)-2*w(2,Ind+1))+w(1,Ind)*w(2,Ind+1));
        dAA(1,3)=dA(1,3)+Ke/dt*(w(1,Ind+1)*(w(3,Ind)-2*w(3,Ind+1))+w(1,Ind)*w(3,Ind+1));
        dAA(2,2)=dA(2,2)+Ke/dt*(w(2,Ind+1)*(w(2,Ind)-2*w(2,Ind+1))+w(2,Ind)*w(2,Ind+1));
        dAA(2,3)=dA(2,3)+Ke/dt*(w(2,Ind+1)*(w(3,Ind)-2*w(3,Ind+1))+w(2,Ind)*w(3,Ind+1));
        dAA(3,3)=dA(3,3)+Ke/dt*(w(3,Ind+1)*(w(3,Ind)-2*w(3,Ind+1))+w(3,Ind)*w(3,Ind+1));
        else
            dAA(1,1)=dA(1,1);
            dAA(1,2)=dA(1,2);
            dAA(1,3)=dA(1,3);
            dAA(2,2)=dA(2,2);
            dAA(2,3)=dA(2,3);
            dAA(3,3)=dA(3,3);
        end
        for ii=1:maxmod
            jj=Ind;
            AA(1,1)=AA(1,1)+wwn(t(Ind+1),t(jj),t(jj+1),w(1,jj),w(1,jj+1),w(1,jj),w(1,jj+1),Ki(ii),Si(ii));
            AA(1,2)=AA(1,2)+wwn(t(Ind+1),t(jj),t(jj+1),w(1,jj),w(1,jj+1),w(2,jj),w(2,jj+1),Ki(ii),Si(ii));
            AA(1,3)=AA(1,3)+wwn(t(Ind+1),t(jj),t(jj+1),w(1,jj),w(1,jj+1),w(3,jj),w(3,jj+1),Ki(ii),Si(ii));
            AA(2,2)=AA(2,2)+wwn(t(Ind+1),t(jj),t(jj+1),w(2,jj),w(2,jj+1),w(2,jj),w(2,jj+1),Ki(ii),Si(ii));
            AA(2,3)=AA(2,3)+wwn(t(Ind+1),t(jj),t(jj+1),w(2,jj),w(2,jj+1),w(3,jj),w(3,jj+1),Ki(ii),Si(ii));
            AA(3,3)=AA(3,3)+wwn(t(Ind+1),t(jj),t(jj+1),w(3,jj),w(3,jj+1),w(3,jj),w(3,jj+1),Ki(ii),Si(ii));
            
            dAA(1,1)=dAA(1,1)+dwwn(t(Ind+1),t(jj),t(jj+1),w(1,jj),w(1,jj+1),w(1,jj),w(1,jj+1),Ki(ii),Si(ii));
            dAA(1,2)=dAA(1,2)+dwwn(t(Ind+1),t(jj),t(jj+1),w(1,jj),w(1,jj+1),w(2,jj),w(2,jj+1),Ki(ii),Si(ii));
            dAA(1,3)=dAA(1,3)+dwwn(t(Ind+1),t(jj),t(jj+1),w(1,jj),w(1,jj+1),w(3,jj),w(3,jj+1),Ki(ii),Si(ii));
            dAA(2,2)=dAA(2,2)+dwwn(t(Ind+1),t(jj),t(jj+1),w(2,jj),w(2,jj+1),w(2,jj),w(2,jj+1),Ki(ii),Si(ii));
            dAA(2,3)=dAA(2,3)+dwwn(t(Ind+1),t(jj),t(jj+1),w(2,jj),w(2,jj+1),w(3,jj),w(3,jj+1),Ki(ii),Si(ii));
            dAA(3,3)=dAA(3,3)+dwwn(t(Ind+1),t(jj),t(jj+1),w(3,jj),w(3,jj+1),w(3,jj),w(3,jj+1),Ki(ii),Si(ii));            
        end
        AA(2,1)=AA(1,2);
        AA(3,1)=AA(1,3);
        AA(3,2)=AA(2,3);
        dAA(2,1)=dAA(1,2);
        dAA(3,1)=dAA(1,3);
        dAA(3,2)=dAA(2,3);
        AA=AA-1/3*eye(3)*trace(AA);
        dAA=(dAA-1/3*eye(3)*trace(dAA))/ka;
        
%         II=AA*R^5/3/GG+E;
%         dII=dAA*R^5/3/GG+dE;
        if Loadtype==1
            II=AA*R^5/3/GG+E;
            dII=dAA*R^5/3/GG+dE;
        else
            II=AA*R^5/3/GG+E*(t(Ind+1)-t0)/End_time;
            dE=E/End_time/ka;
            dII=dAA*R^5/3/GG+dE;
        end
        
           %for testing
        
           
        Q1=RotationM(w(:,Ind+1),2);
        MoI1=Q1'*II*Q1;
        ddI=Q1'*dII*Q1;
        
        if Ind==1 && Iter==1
            I33_old=II(3,3)-E(3,3);
        end
        
        dMoI=MoI1;
        
        dI13=dMoI(1,3)+(MoI1(3,3)+I)*ddI(2,3)/Omega/(MoI1(3,3)-MoI1(2,2))/ka;
        th_new13=dI13/(MoI1(3,3)-MoI1(1,1));
        dI23=dMoI(2,3)-(MoI1(3,3)+I)*ddI(1,3)/Omega/(MoI1(3,3)-MoI1(1,1))/ka;
        th_new23=dI23/(MoI1(3,3)-MoI1(2,2));
        
        th_new33=-(dMoI(3,3)-I33_old)/(dMoI(3,3)+I);
        
        temp_old=temp_v;
%         temp_v=[th_new13,th_new23,sqrt(1-th_new13^2-th_new23^2)]';
        temp_v=[th_new13,th_new23,1+th_new33]';
        

        
%             disp([num2str(Ind) '  ' num2str(Iter)  '    ' num2str(temp_v',15) '  '  num2str(norm(temp_v))]);
        
        Lod(Ind+1)=norm(temp_v);
        
        temp_v=temp_v/Lod(Ind+1);
        temp_v=Q1*temp_v;
        
        
        temp=sort([abs((temp_old(1)-temp_v(1))/temp_v(1))*100 abs((temp_old(2)-temp_v(2))/temp_v(2))*0]);
        if temp(end)~=inf && ~isnan(temp(end))
            Improvement=temp(end);
        elseif temp(end-1)~=inf&& ~isnan(temp(end-1))
            Improvement=temp(end-1);
        else
            Improvement=temp(1);
        end
        w(:,Ind+1)=temp_v*Omega0*Lod(Ind);
        Iter=Iter+1;
    end    
    
    Q=RotationM(w(:,Ind+1),2);
    temp=Q'*[sin(alpha), 0, cos(alpha)]';
    Ls(1,Ind+1)=asin((temp(1)^2+temp(2)^2)^0.5);
    Ls(2,Ind+1)=atan2(temp(2),temp(1));   
    
    disp([num2str(Ind) '  ' num2str(Iter,4)  '  ' num2str(w(:,Ind+1)',6) ' ' num2str(Lod(Ind+1)) '  ' num2str(Ls(:,Ind+1)'*180/pi,6)]);
    Ind=Ind+1;
    
end
t=t(2:Ind)-t(1);
w=w(:,2:Ind);
Lod=Lod(2:Ind);
Ls=Ls(:,2:Ind);
end
function wwns=wwn(t,T0,T1,Wi0,Wi1,Wj0,Wj1,ki,si)
if t>T1
    Es0=exp(si*(t-T0));
    Es1=exp(si*(t-T1));
    dt=T1-T0;
    if abs(si)>1e-7
        wwns=2.*dt.^(-2).*((Es0-Es1)./si).*(ki./si).*(Wi0+(-1).*Wi1).*(Wj0+(-1).*Wj1)./si...
            +si.^(-1).*(Es0.*ki.*Wi0.*Wj0+(-1).*Es1.*ki.*Wi1.*Wj1)+...
            dt.^(-1).*(ki./si).*((Es1.*(Wi1.*(Wj0+(-2).*Wj1)+Wi0.*Wj1)+ ...
            Es0.*(Wi1.*Wj0+Wi0.*((-2).*Wj0+Wj1)))./si);
    else
        b10=(ki./si).*((1/2).*dt.^(-1).*si.*((t+(-1).*T0).^2+(t+(-1).*T1).^2)+dt.^(-2).* si.^(-1).*(2.*dt+2.*(T0+(-1).*T1))+dt.^(-2).*(dt.*(2.*t+(-1).*T0+(-1).*T1)+((-1).*T0+T1).*((-2).*t+T0+T1)));
        b00=(ki./si).*((1/2).*dt.^(-2).*si.*(2.*dt.^2.*(t+(-1).*T0)+(-2).*dt.*(t+(-1).*T0).^2)+(1/2).*si.^2.*(t+(-1).*T0).^2+(1/2).*dt.^(-2).*si.^(-1).*(...
            (-4).*dt+(-4).*(T0+(-1).*T1))+(1/2).*dt.^(-2).*(2.*dt.^2+(-4).* dt.*(t+(-1).*T0)+2.*(T0+(-1).*T1).*((-2).*t+T0+T1)));
        b11=(ki./si).*((-1/2).*dt.^(-2).*si.*(2.*dt.^2.*(t+(-1).*T1)+2.*dt.*(t+(-1).*T1).^2)+(-1/2).*dt.^(-2).*si.^(-1).*(4.*dt+4.*(T0+(-1).*T1))+(-1/2).* ...
            si.^2.*(t+(-1).*T1).^2+(-1/2).*dt.^(-2).*(2.*dt.^2+4.*dt.*(t+(-1).*T1)+(-2).*(T0+(-1).*T1).*((-2).*t+T0+T1)));
        wwns=b00.*Wi0.*Wj0+b10.*(Wi1.*Wj0+Wi0.*Wj1)+b11.*Wi1.*Wj1;
    end
else    
    dt=T1-T0;
    Esi=exp(dt.*si);
    if abs(si*dt(1))>1e-10
        wwns=2.*dt.^(-2).*((-1+Esi)./si)*(ki./si).*((Wi0- Wi1).*(Wj0-Wj1)/si)...
            +(ki./si).*(Esi.*Wi0.*Wj0-Wi1.*Wj1)+...
            dt.^(-1).*(ki./si).*(Wi1.*(Wj0+(-2).*Wj1)+ ...
            Wi0.*Wj1+Esi.*(Wi1.*Wj0+Wi0.*((-2).*Wj0+Wj1)))./si;
    else
        wwns=(ki./si).*(Wi1.*Wj0+Wi0.*Wj1-Wi0.*Wj0-Wi1.*Wj1);
    end
end
end

function dwwns=dwwn(t,T0,T1,Wi0,Wi1,Wj0,Wj1,ki,si)
if t>T1
    dt=T1-T0;
    if abs(si)>1e-10
        dwwns=ki./(si^2*dt.^2).*(exp(-si*T0+t*si).*(Wi1.*((-2+si*dt).*Wj0+2*Wj1)+Wi0.*((2-si*(2-si*dt).*dt).*Wj0...
            +(-2+si*dt).*Wj1))-...
            exp(si*(t-T1)).*(Wi0.*(2*Wj0+(-2-si*dt).*Wj1)+Wi1.*(2*Wj1+(-2-si*dt).*(Wj0-si*dt.*Wj1))));
    else
        Es0=exp(si*(t-T0));
%         Es1=exp(si*(t-T1));
        dwwns=ki.*Es0.*((dt.*si./2).*(Wi1.*Wj0+Wi0.*Wj1)+(1-exp(-dt.*si)-2*dt.*si).* Wi1.*Wj1);
    end
else
    dt=T1-T0;
    Esi=exp(dt.*si);
    if abs(si)>1e-10
        dwwns=Esi.*ki.*Wi0.*Wj0...
            +2.*dt.^(-2).*((-1+Esi)./si).*(ki./si).*(Wi0+(-1).*Wi1).*(Wj0+(-1).*Wj1)...
            +dt.^(-1).*(ki./si).*(Wi1.*(Wj0+(-2).*Wj1)+Wi0.*Wj1+Esi.*(Wi1.*Wj0+Wi0.*((-2).*Wj0+Wj1)));
    else
        dwwns=ki.*(Wi1.*Wj0+Wi0.*Wj1-Wi0.*Wj0);
    end        
end   
end



