function [t, w, Lod,Ls,epsilon]=TPW_Hu_for_Midori6_6(M,alpha,t0,Dt,End_time,Model,Omega_portion,Dt0)
%% Semi-analytical solution for TPW on rotating body whose Chandler period is comparable its dominant relaxation time (Venus-like).
%% by Haiyang Hu   04-13-2017
%% Output
% t: time step;  
% w: colatitude and longitude of the rotational axis (rad) in the body-fixed frame;
% Lod: colatitude and longitude of the mass anomaly (rad) in the bulge-fixed frame; 
% Ls: length of the day change; 
%% Input 
% M: mass of the massanomaly (kg);
% alpha: colatitude (degree) of the mass anomaly. It is placed at 0 degree langutude. 
% t0: Initial Loading time (ka) for the model to reach its desired hydrostatic state. So large enough for all the modes to relax for a model in hydrostatic equilibrium.
% Dt: step size (ka)
% End_time: simulation length (ka)
% Model: choice of the interior model Love numbers;
% Omega_portion: a scale factor for the rotational speed. Set to 1 for  normal simulation.
% Lodtype: 1 for Heaviside, 2 for Ramp type.
% Dt0: initial step_size. Set to be smaller or the same as Dt.
%% Example
% To reproduce figure 5 in [Hu at al, 2017b], type following setings:
% M=-5e18;
% alpha=45;
% End_time=2000;  
% [tt,ww,~,Ls,eps1]=TPW_Hu_for_Midori6_6(M,alpha,1e7,1,End_time,0,1,0.01);

%% Load Love numbers for the chosen model
if Model==0
    data=dlmread('F:\Python_code\Normal Mode\Venus0_0.txt');  
else
    error('Wrong type of model.');
end
% maxdeg=data(1);
nlayer=data(2);
maxmod=data(3);
data=data(4+nlayer:end,:);
D=1;

Ke=data((maxmod+2)*(D-1)+1,3);
Kf=data((maxmod+2)*(D-1)+maxmod+2,3);
Ki=-data((maxmod+2)*(D-1)+(1:maxmod)+1,3).*data((maxmod+2)*(D-1)+(1:maxmod)+1,2);
Si=data((maxmod+2)*(D-1)+(1:maxmod)+1,2);

%%
GG=6.67384e-11;
R=6052e3;
Omega=2.99e-7*Omega_portion;
Omega0=Omega;
ka=1000*365.25*24*3600;

I=5.9408e+37;
 
%% Load parameters
alpha=alpha/180*pi;
x=sin(alpha); y=0; z=cos(alpha);
E=M*R^2*[y^2+z^2 -x*y -x*z; -y*x x^2+z^2 -y*z; -z*x -z*y x^2+y^2]; % Load matrix    
dE=0;
% dE0=E/Dt0;

%%
t=zeros(1,3000);
t(1)=t0;
w=zeros(3,3000);
w(:,1)=[0,0,Omega]';
Lod=zeros(1,1000);
Lod(1)=1;

Ls=zeros(2,3000);
Ls(:,1)=[alpha,0];

epsilon=zeros(1,3000);

A=zeros(3);
dA=zeros(3);
Ind=1;


Iter_old=100;
while t(Ind)<t0+End_time
%     if Ind>1
%         break;
%     end
%     if t(Ind)>1e4+t0  %remove the mass anomaly after 1 milion year.
%        E=zeros(3);
%     end
    if Ind==1
        t(Ind+1)=t(Ind)+Dt0;
    else
         t(Ind+1)=t(Ind)+Dt;
    end
    if Ind<0
        w(:,Ind+1)=2*w(:,Ind)*Lod(Ind)-w(:,Ind-1)*Lod(Ind-1);
    else
    w(:,Ind+1)=w(:,Ind)*Lod(Ind);
    end
    Coe=Ki./Si.*(exp(t(Ind+1)*Si)-exp((t(Ind+1)-t(1))*Si));
    Coe=sum(Coe);
    A(1,1)=Coe*w(1,1)*w(1,1);
    A(1,2)=Coe*w(1,1)*w(2,1);
    A(1,3)=Coe*w(1,1)*w(3,1);
    A(2,2)=Coe*w(2,1)*w(2,1);
    A(2,3)=Coe*w(2,1)*w(3,1);
    A(3,3)=Coe*w(3,1)*w(3,1);  
    
    Coe=Ki.*(exp(t(Ind+1)*Si)-exp((t(Ind+1)-t(1))*Si));
    Coe=sum(Coe);
    if Ind~=0
        dt=t(Ind+1)-t(Ind);
        dA(1,1)=Coe*w(1,1)*w(1,1)+Ke/dt*(w(1,Ind+1)*(w(1,Ind)-2*w(1,Ind+1))+w(1,Ind)*w(1,Ind+1));
        dA(1,2)=Coe*w(1,1)*w(2,1)+Ke/dt*(w(1,Ind+1)*(w(2,Ind)-2*w(2,Ind+1))+w(1,Ind)*w(2,Ind+1));
        dA(1,3)=Coe*w(1,1)*w(3,1)+Ke/dt*(w(1,Ind+1)*(w(3,Ind)-2*w(3,Ind+1))+w(1,Ind)*w(3,Ind+1));
        dA(2,2)=Coe*w(2,1)*w(2,1)+Ke/dt*(w(2,Ind+1)*(w(2,Ind)-2*w(2,Ind+1))+w(2,Ind)*w(2,Ind+1));
        dA(2,3)=Coe*w(2,1)*w(3,1)+Ke/dt*(w(2,Ind+1)*(w(3,Ind)-2*w(3,Ind+1))+w(2,Ind)*w(3,Ind+1));
        dA(3,3)=Coe*w(3,1)*w(3,1)+Ke/dt*(w(3,Ind+1)*(w(3,Ind)-2*w(3,Ind+1))+w(3,Ind)*w(3,Ind+1));
    else
        dA(1,1)=0;
        dA(1,2)=0;
        dA(1,3)=0;
        dA(2,2)=0;
        dA(2,3)=0;
        dA(3,3)=0;
    end
    for ii=1:maxmod
        if Ind>1
            temp=wwn(t(Ind+1),t(1:Ind-1),t(2:Ind),w(1,1:Ind-1),w(1,2:Ind),w(1,1:Ind-1),w(1,2:Ind),Ki(ii),Si(ii));
            A(1,1)=A(1,1)+sum(temp);
            temp=wwn(t(Ind+1),t(1:Ind-1),t(2:Ind),w(1,1:Ind-1),w(1,2:Ind),w(2,1:Ind-1),w(2,2:Ind),Ki(ii),Si(ii));
            A(1,2)=A(1,2)+sum(temp);
            temp=wwn(t(Ind+1),t(1:Ind-1),t(2:Ind),w(1,1:Ind-1),w(1,2:Ind),w(3,1:Ind-1),w(3,2:Ind),Ki(ii),Si(ii));
            A(1,3)=A(1,3)+sum(temp);
            temp=wwn(t(Ind+1),t(1:Ind-1),t(2:Ind),w(2,1:Ind-1),w(2,2:Ind),w(2,1:Ind-1),w(2,2:Ind),Ki(ii),Si(ii));
            A(2,2)=A(2,2)+sum(temp);
            temp=wwn(t(Ind+1),t(1:Ind-1),t(2:Ind),w(2,1:Ind-1),w(2,2:Ind),w(3,1:Ind-1),w(3,2:Ind),Ki(ii),Si(ii));
            A(2,3)=A(2,3)+sum(temp);
            temp=wwn(t(Ind+1),t(1:Ind-1),t(2:Ind),w(3,1:Ind-1),w(3,2:Ind),w(3,1:Ind-1),w(3,2:Ind),Ki(ii),Si(ii));
            A(3,3)=A(3,3)+sum(temp);
            
            temp=dwwn(t(Ind+1),t(1:Ind-1),t(2:Ind),w(1,1:Ind-1),w(1,2:Ind),w(1,1:Ind-1),w(1,2:Ind),Ki(ii),Si(ii));
            dA(1,1)=dA(1,1)+sum(temp);
            temp=dwwn(t(Ind+1),t(1:Ind-1),t(2:Ind),w(1,1:Ind-1),w(1,2:Ind),w(2,1:Ind-1),w(2,2:Ind),Ki(ii),Si(ii));
            dA(1,2)=dA(1,2)+sum(temp);
            temp=dwwn(t(Ind+1),t(1:Ind-1),t(2:Ind),w(1,1:Ind-1),w(1,2:Ind),w(3,1:Ind-1),w(3,2:Ind),Ki(ii),Si(ii));
            dA(1,3)=dA(1,3)+sum(temp);
            temp=dwwn(t(Ind+1),t(1:Ind-1),t(2:Ind),w(2,1:Ind-1),w(2,2:Ind),w(2,1:Ind-1),w(2,2:Ind),Ki(ii),Si(ii));
            dA(2,2)=dA(2,2)+sum(temp);
            temp=dwwn(t(Ind+1),t(1:Ind-1),t(2:Ind),w(2,1:Ind-1),w(2,2:Ind),w(3,1:Ind-1),w(3,2:Ind),Ki(ii),Si(ii));
            dA(2,3)=dA(2,3)+sum(temp);
            temp=dwwn(t(Ind+1),t(1:Ind-1),t(2:Ind),w(3,1:Ind-1),w(3,2:Ind),w(3,1:Ind-1),w(3,2:Ind),Ki(ii),Si(ii));
            dA(3,3)=dA(3,3)+sum(temp);
        end
    end

    
    
    AA=zeros(3);
    dAA=zeros(3);

    Improvement=100;
    Converge=1e-8;

    Improvement2=100;
    
    temp_v=zeros(3,1);
    Iter=1;
    
    
    
    Mid_store=zeros(3,3000);
    F1=[1,0,0];
    F2=[1,0,0];    
    IterT=1;
    
    while Improvement>Converge && IterT<1e4
        AA(1,1)=A(1,1)+Ke*w(1,Ind+1)*w(1,Ind+1);
        AA(1,2)=A(1,2)+Ke*w(1,Ind+1)*w(2,Ind+1);
        AA(1,3)=A(1,3)+Ke*w(1,Ind+1)*w(3,Ind+1);
        AA(2,2)=A(2,2)+Ke*w(2,Ind+1)*w(2,Ind+1);
        AA(2,3)=A(2,3)+Ke*w(2,Ind+1)*w(3,Ind+1);
        AA(3,3)=A(3,3)+Ke*w(3,Ind+1)*w(3,Ind+1);
        
        dt=t(Ind+1)-t(Ind);
        if Ind~=0
        dAA(1,1)=dA(1,1)+Ke/dt*(w(1,Ind+1)*(w(1,Ind)-2*w(1,Ind+1))+w(1,Ind)*w(1,Ind+1));
        dAA(1,2)=dA(1,2)+Ke/dt*(w(1,Ind+1)*(w(2,Ind)-2*w(2,Ind+1))+w(1,Ind)*w(2,Ind+1));
        dAA(1,3)=dA(1,3)+Ke/dt*(w(1,Ind+1)*(w(3,Ind)-2*w(3,Ind+1))+w(1,Ind)*w(3,Ind+1));
        dAA(2,2)=dA(2,2)+Ke/dt*(w(2,Ind+1)*(w(2,Ind)-2*w(2,Ind+1))+w(2,Ind)*w(2,Ind+1));
        dAA(2,3)=dA(2,3)+Ke/dt*(w(2,Ind+1)*(w(3,Ind)-2*w(3,Ind+1))+w(2,Ind)*w(3,Ind+1));
        dAA(3,3)=dA(3,3)+Ke/dt*(w(3,Ind+1)*(w(3,Ind)-2*w(3,Ind+1))+w(3,Ind)*w(3,Ind+1));
        else
            dAA(1,1)=dA(1,1);
            dAA(1,2)=dA(1,2);
            dAA(1,3)=dA(1,3);
            dAA(2,2)=dA(2,2);
            dAA(2,3)=dA(2,3);
            dAA(3,3)=dA(3,3);
        end
        for ii=1:maxmod
            jj=Ind;
            AA(1,1)=AA(1,1)+wwn(t(Ind+1),t(jj),t(jj+1),w(1,jj),w(1,jj+1),w(1,jj),w(1,jj+1),Ki(ii),Si(ii));
            AA(1,2)=AA(1,2)+wwn(t(Ind+1),t(jj),t(jj+1),w(1,jj),w(1,jj+1),w(2,jj),w(2,jj+1),Ki(ii),Si(ii));
            AA(1,3)=AA(1,3)+wwn(t(Ind+1),t(jj),t(jj+1),w(1,jj),w(1,jj+1),w(3,jj),w(3,jj+1),Ki(ii),Si(ii));
            AA(2,2)=AA(2,2)+wwn(t(Ind+1),t(jj),t(jj+1),w(2,jj),w(2,jj+1),w(2,jj),w(2,jj+1),Ki(ii),Si(ii));
            AA(2,3)=AA(2,3)+wwn(t(Ind+1),t(jj),t(jj+1),w(2,jj),w(2,jj+1),w(3,jj),w(3,jj+1),Ki(ii),Si(ii));
            AA(3,3)=AA(3,3)+wwn(t(Ind+1),t(jj),t(jj+1),w(3,jj),w(3,jj+1),w(3,jj),w(3,jj+1),Ki(ii),Si(ii));
            
            dAA(1,1)=dAA(1,1)+dwwn(t(Ind+1),t(jj),t(jj+1),w(1,jj),w(1,jj+1),w(1,jj),w(1,jj+1),Ki(ii),Si(ii));
            dAA(1,2)=dAA(1,2)+dwwn(t(Ind+1),t(jj),t(jj+1),w(1,jj),w(1,jj+1),w(2,jj),w(2,jj+1),Ki(ii),Si(ii));
            dAA(1,3)=dAA(1,3)+dwwn(t(Ind+1),t(jj),t(jj+1),w(1,jj),w(1,jj+1),w(3,jj),w(3,jj+1),Ki(ii),Si(ii));
            dAA(2,2)=dAA(2,2)+dwwn(t(Ind+1),t(jj),t(jj+1),w(2,jj),w(2,jj+1),w(2,jj),w(2,jj+1),Ki(ii),Si(ii));
            dAA(2,3)=dAA(2,3)+dwwn(t(Ind+1),t(jj),t(jj+1),w(2,jj),w(2,jj+1),w(3,jj),w(3,jj+1),Ki(ii),Si(ii));
            dAA(3,3)=dAA(3,3)+dwwn(t(Ind+1),t(jj),t(jj+1),w(3,jj),w(3,jj+1),w(3,jj),w(3,jj+1),Ki(ii),Si(ii));            
        end
        AA(2,1)=AA(1,2);
        AA(3,1)=AA(1,3);
        AA(3,2)=AA(2,3);
        dAA(2,1)=dAA(1,2);
        dAA(3,1)=dAA(1,3);
        dAA(3,2)=dAA(2,3);
        AA=AA-1/3*eye(3)*trace(AA);
        dAA=(dAA-1/3*eye(3)*trace(dAA))/ka;
        
        if Ind>=1
            MOI_old=AA*R^5/3/GG;
        end
        
        II=MOI_old+E;   
%          II=E;
        
        Q1=RotationM2(w(:,Ind),[x,y,z]',2);
%         Q1=RotationM(w(:,Ind+1),2);
        MoI1=Q1'*II*Q1;

        dII=dAA*R^5/3/GG+dE;
%           dII=0;
        
        ddI=Q1'*dII*Q1;
        
        
        
        
        
        if Ind==1 && Iter==1            
            I33_old=II(3,3)-E(3,3);
        end
        if Ind==1
            DDt=Dt0;
        else
            DDt=Dt;
        end
        dMoI=MoI1;
        IA=dMoI(1,1)+I;
        IB=dMoI(2,2)+I;
        IC=dMoI(3,3)+I;

        t13=0;
        t23=0;
        for repeat=1:7
            t13=1/2*(DDt)*ka/(IA)*(Omega*(dMoI(2,3)-ddI(2,3)*DDt)-ddI(1,3)/ka-(IC-IB)*Omega*t23);  %
            t23=-(DDt)*ka/(IB)*(Omega*(dMoI(1,3)-ddI(1,3)*DDt)+ddI(2,3)/ka+(IC-IA)*Omega*t13); %
        end
        th_new13=t13;
        th_new23=t23;        
%         th_new13=(DDt)*ka/(dMoI(1,1)+I)*(Omega*(dMoI(2,3)-ddI(2,3)*DDt)-ddI(1,3)/ka);  %
%         th_new23=-(DDt)*ka/(dMoI(2,2)+I)*(Omega*(dMoI(1,3)-ddI(1,3)*DDt)+ddI(2,3)/ka); %
        th_new33=-(dMoI(3,3)-I33_old)/(dMoI(3,3)+I);
        
        temp_old=temp_v;
%%
        mido=[th_new13,th_new23,(1-th_new13^2-th_new23^2)^0.5]';
        temp_v=[0,0,0]'+mido*1;
        
        Lod(Ind+1)=1+th_new33*0;
        
        temp_v=temp_v/Lod(Ind+1);
        temp_v=Q1*temp_v;
        Mid_store(:,IterT)=temp_v;
        
        temp=sort([abs((temp_old(1)-temp_v(1))/temp_v(1))*100*(temp_v(1)/temp_v(2))^0.1 abs((temp_old(2)-temp_v(2))/temp_v(2)*(temp_v(2)/temp_v(1))^0.1)*100]);
        if temp(end)~=inf && ~isnan(temp(end))
            Improvement=temp(end);
        elseif temp(end-1)~=inf&& ~isnan(temp(end-1))
            Improvement=temp(end-1);
        else
            Improvement=temp(1);
        end
%           Improvement2=acos(sum(temp_old.*temp_v)); 
        w(:,Ind+1)=temp_v*Omega0*Lod(Ind);
        Iter=Iter+1;
        IterT=IterT+1;
    end    
    
%     AA_old=AA;
    Q=RotationM(w(:,Ind+1),2);
    temp=Q'*[sin(alpha), 0, cos(alpha)]';
    Ls(1,Ind+1)=asin((temp(1)^2+temp(2)^2)^0.5);
    Ls(2,Ind+1)=atan2(temp(2),temp(1));    
    
     disp([num2str(Ind) '  ' num2str(IterT,4)  '  ' num2str(w(:,Ind+1)',6) ' ' num2str(Lod(Ind+1)) '  ' num2str(Ls(:,Ind+1)'*180/pi,6) '   ' num2str(MOI_old(1,3)/E(1,3))]);
%     Body=AA*R^5/3/GG;
    [EIG,~]=eig(II);
    epsilon(Ind)=acos(sum(EIG(:,3).*temp_v))*180/pi;
    if epsilon(Ind)>90
       epsilon(Ind)=180-epsilon(Ind); 
    end
    Ind=Ind+1;
    Iter_old=IterT;
end
epsilon=epsilon(1:Ind-1);
t=t(2:Ind)-t(1);
w=w(:,2:Ind);
Lod=Lod(2:Ind);
Ls=Ls(:,2:Ind);
end

function wwns=wwn(t,T0,T1,Wi0,Wi1,Wj0,Wj1,ki,si)
if t>T1
    Es0=exp(si*(t-T0));
    Es1=exp(si*(t-T1));
    dt=T1-T0;
    if abs(si)>1e-5
    wwns=2.*dt.^(-2).*((Es0-Es1)./si).*(ki./si).*(Wi0+(-1).*Wi1).*(Wj0+(-1).*Wj1)./si...
        +si.^(-1).*(Es0.*ki.*Wi0.*Wj0+(-1).*Es1.*ki.*Wi1.*Wj1)+...
        dt.^(-1).*(ki./si).*((Es1.*(Wi1.*(Wj0+(-2).*Wj1)+Wi0.*Wj1)+ ...
        Es0.*(Wi1.*Wj0+Wi0.*((-2).*Wj0+Wj1)))./si);
    else 
        b10=(ki./si).*((1/2).*dt.^(-1).*si.*((t+(-1).*T0).^2+(t+(-1).*T1).^2)+dt.^(-2).* si.^(-1).*(2.*dt+2.*(T0+(-1).*T1))+dt.^(-2).*(dt.*(2.*t+(-1).*T0+(-1).*T1)+((-1).*T0+T1).*((-2).*t+T0+T1)));
        b00=(ki./si).*((1/2).*dt.^(-2).*si.*(2.*dt.^2.*(t+(-1).*T0)+(-2).*dt.*(t+(-1).*T0).^2)+(1/2).*si.^2.*(t+(-1).*T0).^2+(1/2).*dt.^(-2).*si.^(-1).*(...
            (-4).*dt+(-4).*(T0+(-1).*T1))+(1/2).*dt.^(-2).*(2.*dt.^2+(-4).* dt.*(t+(-1).*T0)+2.*(T0+(-1).*T1).*((-2).*t+T0+T1)));
        b11=(ki./si).*((-1/2).*dt.^(-2).*si.*(2.*dt.^2.*(t+(-1).*T1)+2.*dt.*(t+(-1).*T1).^2)+(-1/2).*dt.^(-2).*si.^(-1).*(4.*dt+4.*(T0+(-1).*T1))+(-1/2).* ...
            si.^2.*(t+(-1).*T1).^2+(-1/2).*dt.^(-2).*(2.*dt.^2+4.*dt.*(t+(-1).*T1)+(-2).*(T0+(-1).*T1).*((-2).*t+T0+T1)));
        wwns=b00.*Wi0.*Wj0+b10.*(Wi1.*Wj0+Wi0.*Wj1)+b11.*Wi1.*Wj1;
    end
else    
    dt=T1-T0;
    Esi=exp(dt.*si);
    if abs(si)>1e-10
        wwns=2.*dt.^(-2).*((-1+Esi)./si)*(ki./si).*((Wi0- Wi1).*(Wj0-Wj1)/si)...
            +(ki./si).*(Esi.*Wi0.*Wj0-Wi1.*Wj1)+...
            dt.^(-1).*(ki./si).*(Wi1.*(Wj0+(-2).*Wj1)+ ...
            Wi0.*Wj1+Esi.*(Wi1.*Wj0+Wi0.*((-2).*Wj0+Wj1)))./si;
    else
        wwns=(ki./si).*(Wi1.*Wj0+Wi0.*Wj1-Wi0.*Wj0-Wi1.*Wj1);
    end
end
end

function dwwns=dwwn(t,T0,T1,Wi0,Wi1,Wj0,Wj1,ki,si)
if t>T1
    dt=T1-T0;
    if abs(si)>1e-10
        dwwns=ki./(si^2*dt.^2).*(exp(-si*T0+t*si).*(Wi1.*((-2+si*dt).*Wj0+2*Wj1)+Wi0.*((2-si*(2-si*dt).*dt).*Wj0...
            +(-2+si*dt).*Wj1))-...
            exp(si*(t-T1)).*(Wi0.*(2*Wj0+(-2-si*dt).*Wj1)+Wi1.*(2*Wj1+(-2-si*dt).*(Wj0-si*dt.*Wj1))));
    else
        Es0=exp(si*(t-T0));
%         Es1=exp(si*(t-T1));
        dwwns=ki.*Es0.*((dt.*si./2).*(Wi1.*Wj0+Wi0.*Wj1)+(1-exp(-dt.*si)-2*dt.*si).* Wi1.*Wj1);
    end
else
    dt=T1-T0;
    Esi=exp(dt.*si);
    if abs(si)>1e-10
        dwwns=Esi.*ki.*Wi0.*Wj0...
            +2.*dt.^(-2).*((-1+Esi)./si).*(ki./si).*(Wi0+(-1).*Wi1).*(Wj0+(-1).*Wj1)...
            +dt.^(-1).*(ki./si).*(Wi1.*(Wj0+(-2).*Wj1)+Wi0.*Wj1+Esi.*(Wi1.*Wj0+Wi0.*((-2).*Wj0+Wj1)));
    else
        dwwns=ki.*(Wi1.*Wj0+Wi0.*Wj1-Wi0.*Wj0);
    end        
end   
end


